@@ -1,4 +1,36 @@
-"""Import magnettech 1D and 2D datasets from its xml file(s)."""
+"""
+Importer for the Magnettech XML format.
+
+Magnettech spectrometers can write data to either XML or CSV files.
+Currently, only the XML file format is supported. Generally each individual
+scan gets saved into its own file, but the averaged data stored in an
+additional file with ``_result`` suffixed to its base name.
+
+Two-dimensional datasets, be it from modifying microwave power (power
+sweep), modulation amplitude, or goniometer angle, are stored as well in
+individual files per second parameter. Additionally, if you perform multiple
+averages per parameter value (microwave power, modulation amplitude, goniometer
+angle, ...) you will end up with directories for each of these and again
+additionally a file with the averaged data.
+
+A slight complication with the way Magnettech spectrometers obtain their
+data is the rather high magnetic field sampling frequency (typically 10^4
+points) and the non-equidistant field axis. The latter is unique for each
+individual measurement in terms of the number of points, grid, and start and
+end value. Despite the field range set in the software, the spectrometer
+typically records the spectra slightly broader.
+
+For two-dimensional datasets, all this means that the data for the
+individual traces have to be interpolated to a common axis before a
+two-dimensional matrix can be constructed. As the microwave frequency is
+recorded for each individual trace, a frequency correction can be applied
+beforehand.
+
+Currently, only one-dimensional datasets and angular-dependent measurements
+(goniometer sweeps) can be imported. Implementing importers for other types
+of two-dimensional datasets is planned for the future.
+
+"""
 import base64
 import glob
 import os
@@ -18,13 +50,13 @@ import cwepr.processing
 import cwepr.exceptions
 
 
-class MagnettechXmlImporter(aspecd.io.DatasetImporter):
+class MagnettechXMLImporter(aspecd.io.DatasetImporter):
     """Import cw-EPR raw data from the Magnettech benchtop spectrometer.
 
     Magnettech provides a XML-file with the results. Specialities of this
-    format are existing and will be briefly explained: The data is coded in
-    hex-numbers, and the y axis consists of 10 times more points than the
-    y-axis. Therefore, an interpolation is needed to expand the axis to the
+    format are existing and will be briefly explained: The data is encoded in
+    hex numbers, and the *y* axis consists of 10 times more points than the
+    *y* axis. Therefore, an interpolation is needed to expand the axis to the
     necessary amount of points.
 
 
@@ -33,8 +65,14 @@ class MagnettechXmlImporter(aspecd.io.DatasetImporter):
     root: :class:`str`
         path of the root directory
 
-    .. todo::
-        Dokumentation der Attribute, Sortieren der Funktionen.
+    full_filename: :class:`str`
+        Filename with file extension
+
+    load_infofile: :class:`bool`
+        Skips import of infofile if set to False.
+
+    xml_metadata: :class:`dict`
+        Metadata from xml file, eventually imported to metadata.
 
     """
 
@@ -43,10 +81,10 @@ class MagnettechXmlImporter(aspecd.io.DatasetImporter):
             source = source[:-4]
         super().__init__(source=source)
         # public properties
-        self.load_infofile = True
         self.root = None
-        self.xml_metadata = dict()
         self.full_filename = ''
+        self.load_infofile = True
+        self.xml_metadata = dict()
         # private properties
         self._infofile = aspecd.infofile.Infofile()
         self._bfrom = float()
@@ -88,6 +126,7 @@ class MagnettechXmlImporter(aspecd.io.DatasetImporter):
     def _get_raw_data(self):
         self._xvalues = \
             self._convert_base64string_to_np_array(self.root[0][0][1][0].text)
+        # TODO: Not take last element but first harmonic (or so)
         self._yvalues = \
             self._convert_base64string_to_np_array(self.root[0][0][1][-1].text)
 
@@ -172,10 +211,7 @@ class MagnettechXmlImporter(aspecd.io.DatasetImporter):
         mapper = aspecd.metadata.MetadataMapper()
         mapper.version = infofile_version
         mapper.metadata = self._infofile.parameters
-        root_path = \
-            os.path.split(os.path.split(os.path.abspath(__file__))[0])[0]
-        mapper.recipe_filename = os.path.join(
-            root_path, 'metadata_mapper_cwepr.yaml')
+        mapper.recipe_filename = 'cwepr@metadata_mapper_cwepr.yaml'
         mapper.map()
         self.dataset.metadata.from_dict(mapper.metadata)
 
@@ -247,18 +283,25 @@ class MagnettechXmlImporter(aspecd.io.DatasetImporter):
 
 
 class GoniometerSweepImporter(aspecd.io.DatasetImporter):
-    """Import angular dependent data from Magnettech benchtop spectrometer.
+    """Import-angular dependent data from Magnettech benchtop spectrometer.
 
-    ..note::
+    .. note::
         Metadata are only taken from the infofile, ignoring the (much likely
         more accurate) xml-file metadata.
+
+    Attributes
+    ----------
+    load_infofile: :class:`bool`
+        Skips import of infofile if set to False.
+
     """
 
     def __init__(self, source=''):
         super().__init__(source=source)
-        self._infofile = aspecd.infofile.Infofile()
         self.dataset = cwepr.dataset.ExperimentalDataset()
         self.filenames = None
+        self.load_infofile = True
+        self._infofile = aspecd.infofile.Infofile()
         self._data = None
         self._angles = []
 
@@ -290,7 +333,7 @@ class GoniometerSweepImporter(aspecd.io.DatasetImporter):
         # import all files without infofile
         for num, filename in enumerate(self.filenames):
             filename = filename[:-4]  # remove extension
-            importer = cwepr.io.MagnettechXmlImporter(source=filename)
+            importer = cwepr.io.MagnettechXMLImporter(source=filename)
             importer.load_infofile = False
             self._data.append(cwepr.dataset.ExperimentalDataset())
             self._data[num].import_from(importer)
@@ -338,8 +381,19 @@ class GoniometerSweepImporter(aspecd.io.DatasetImporter):
             All information from the xml-file are completely ignored.
 
         """
-        self._load_infofile()
-        self._map_infofile()
+        # First import metadata from infofile, then override hand-written
+        # information by metadata from xml-file. The order matters.
+        if self.load_infofile and self._infofile_exists():
+            self._load_infofile()
+            self._map_infofile()
+
+    def _infofile_exists(self):
+        if self._get_infofile_name() and os.path.exists(
+                self._get_infofile_name()[0]):
+            return True
+        print('No infofile found for dataset %s, import continued without '
+              'infofile.' % os.path.split(self.source)[1])
+        return False
 
     def _load_infofile(self):
         """Import infofile and parse it."""
@@ -350,8 +404,10 @@ class GoniometerSweepImporter(aspecd.io.DatasetImporter):
         self._infofile.parse()
 
     def _get_infofile_name(self):
-        folder_path = os.path.split(self.source)[0]
-        return glob.glob(folder_path + '.info')
+        if self.source.endswith('/'):
+            folder_path = os.path.split(self.source)[0]
+            return glob.glob(folder_path + '.info')
+        return glob.glob(self.source + '.info')
 
     def _assign_comment_as_annotation(self):
         comment = aspecd.annotation.Comment()
@@ -363,10 +419,7 @@ class GoniometerSweepImporter(aspecd.io.DatasetImporter):
         mapper = aspecd.metadata.MetadataMapper()
         mapper.version = infofile_version
         mapper.metadata = self._infofile.parameters
-        root_path = \
-            os.path.split(os.path.split(os.path.abspath(__file__))[0])[0]
-        mapper.recipe_filename = os.path.join(
-            root_path, 'metadata_mapper_cwepr.yaml')
+        mapper.recipe_filename = 'cwepr@metadata_mapper_cwepr.yaml'
         mapper.map()
         self.dataset.metadata.from_dict(mapper.metadata)
         self._convert_values_to_strings()
